<?php

namespace SGalinski\SgJobs\Hooks\PageLayoutView;

/***************************************************************
 *  Copyright notice
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *  All rights reserved
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Backend\View\PageLayoutView;
use TYPO3\CMS\Core\Localization\LanguageService;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * Class PluginRenderer - Renders backend previews for plugins.
 *
 * @package SGalinski\SgJobs\Hooks\PageLayoutView
 */
class PluginRenderer implements \TYPO3\CMS\Backend\View\PageLayoutViewDrawItemHookInterface {

	/**
	 * @inheritDoc
	 * @noinspection ReferencingObjectsInspection
	 */
	public function preProcess(
		PageLayoutView &$parentObject, &$drawItem, &$headerContent, &$itemContent, array &$row
	): void {
		switch ($row['list_type']) {
			case 'sgjobs_jobapplication':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('JobApplication');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['sDEF']['lDEF'];

				$redirectPage = $pluginConfiguration['settings.redirectPage']['vDEF'];
				$templateData = [
					'redirectPage' => is_string($redirectPage) ? $this->addFieldContentsToRecordIdList(
						'pages', $redirectPage
					) : '',
					'privacyPolicyPage' => $pluginConfiguration['settings.privacyPolicyPage']['vDEF']
				];

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			case 'sgjobs_joblist':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('Joblist');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['sDEF']['lDEF'];

				$applyPage = $pluginConfiguration['settings.applyPage']['vDEF'];
				$filterByExperienceLevel = $pluginConfiguration['settings.filterByExperienceLevel']['vDEF'];
				$templateData = [
					'applyPage' => is_string($applyPage) ? $this->addFieldContentsToRecordIdList(
						'pages', $applyPage
					) : '',
					'jobLimit' => $pluginConfiguration['settings.jobLimit']['vDEF'],
					'orderBy' => $pluginConfiguration['settings.orderBy']['vDEF'],
					'filterByExperienceLevel' => is_string(
						$filterByExperienceLevel
					) ? $this->addFieldContentsToRecordIdList(
						'tx_sgjobs_domain_model_experience_level', $filterByExperienceLevel
					) : ''
				];

				$view->assign('data', $templateData);

				// Also assign the standard plugin settings to the template
				// as they are actually used in the corresponding controller action.
				$view->assign('storagePages', is_string($row['pages']) ? $this->addFieldContentsToRecordIdList(
					'pages', $row['pages']
				) : '');
				$view->assign('recursive', $row['recursive']);

				$itemContent .= $view->render();
				break;

			case 'sgjobs_jobteaser':
				$drawItem = FALSE;

				$view = $this->createViewWithTemplate('JobTeaser');
				$view->assign('uid', $row['uid']);

				$this->adaptPluginHeaderContent($headerContent, $row);

				// Get available plugin settings and their values from flexform
				$pluginConfiguration = GeneralUtility::xml2array(
					$row['pi_flexform'], 'T3DataStructure'
				)['data']['sDEF']['lDEF'];

				$offersPage = $pluginConfiguration['settings.offersPage']['vDEF'];
				$applyPage = $pluginConfiguration['settings.applyPage']['vDEF'];
				$locations = $pluginConfiguration['settings.locations']['vDEF'];
				$templateData = [
					'offersPage' => is_string($offersPage) ? $this->addFieldContentsToRecordIdList(
						'pages', $offersPage
					) : '',
					'applyPage' => is_string($applyPage) ? $this->addFieldContentsToRecordIdList(
						'pages', $applyPage
					) : '',
					'locations' => is_string($locations) ? $this->addFieldContentsToRecordIdList(
						'tx_sgjobs_domain_model_company', $locations, 'name'
					) : '',
				];

				$view->assign('data', $templateData);

				$itemContent .= $view->render();
				break;

			default:
				// No need to do anything
		}
	}


	/**
	 * Creates a new StandaloneView object with template and partial root paths,
	 * attaches the template with the given name to it and returns it.
	 *
	 * @param string $templateName
	 * @return StandaloneView
	 */
	protected
	function createViewWithTemplate(string $templateName): StandaloneView {
		$view = GeneralUtility::makeInstance(StandaloneView::class);
		$view->setTemplateRootPaths(['EXT:sg_jobs/Resources/Private/Templates/Backend']);
		$view->setPartialRootPaths(['EXT:sg_jobs/Resources/Private/Partials/Backend']);
		if (!str_ends_with($templateName, '.html')) {
			$templateName .= '.html';
		}
		$view->setTemplate($templateName);
		return $view;
	}

	/**
	 * Adapts the given $headerContent.
	 * To be used in all plugin previews so the Header Contents appear similarly.
	 *
	 * @param $headerContent
	 * @param $row
	 */
	protected
	function adaptPluginHeaderContent(&$headerContent, $row): void {
		$headerContent = '<h4>' . $this->getPluginNameForHeaderContent(
				(int) $row['pid'], $row['list_type']
			) . $headerContent . '</h4>';
	}

	/**
	 * Finds the label of the given $listType element on the page with the given $pid
	 * and returns it wrapped for use in the backend preview's header content.
	 *
	 * @param int $pid
	 * @param string $listType
	 * @return string
	 */
	protected
	function getPluginNameForHeaderContent(int $pid, string $listType): string {
		$languageService = GeneralUtility::makeInstance(LanguageService::class);

		$pluginName = $languageService->sL(
			BackendUtility::getLabelFromItemListMerged(
				$pid, 'tt_content', 'list_type', $listType
			)
		);
		return '<span class="label label-primary">' . $pluginName . '</span>&nbsp;';
	}

	/**
	 * Takes a comma-separated list of record IDs, the corresponding table and optionally the field to look up.
	 * Returns another comma-space-separated list of the same records with the content of the field added.
	 * The returned list should look nice enough to be rendered in the backend preview directly.
	 *
	 * @param string $table
	 * @param string $recordIdList
	 * @param string $field
	 * @return string
	 */
	protected function addFieldContentsToRecordIdList(
		string $table, string $recordIdList, string $field = 'title'
	): string {
		$recordIdsArray = GeneralUtility::intExplode(',', $recordIdList, TRUE);
		$recordsWithTitlesArray = [];

		foreach ($recordIdsArray as $recordId) {
			$recordsWithTitlesArray[] = BackendUtility::getRecord(
					$table, $recordId, $field
				)[$field] . ' [' . $recordId . ']';
		}
		return implode(', ', $recordsWithTitlesArray);
	}
}
